--[[
Author: Ayantir & Baertram
Filename: RollingStones
Version: K-DST
]]--

local db = {}

local myAddon = {
	name					= "RollingStones",
	author				= "Ayantir & Baertram",
	savedVarsName		= "ROLLINGSTONES",
	savedVarsVersion	= 1.2,
	optionsVersion    = "K-DST",
	website				= "http://www.esoui.com/downloads/info1186-TheRollingTelVarStones.html",
}

local actualDistrictBonus = 0
local playerAlliance = GetUnitAlliance("player")
local ICKeeps = {
	[141] = true, -- "Nobles District",
	[142] = true, -- "Memorial District",
	[143] = true, -- "Arboretum",
	[146] = true, -- "Arena District",
	[147] = true, -- "Temple District",
	[148] = true, -- "Elven Gardens District",
}

local function RollingStones()
   
	--Get the current number of Tel'Var stones in your inventory
	local currentTelvarStonesInInventory = GetCarriedCurrencyAmount(CURT_TELVAR_STONES)
    --Get the currently banked Tel'Var stones
   local currentTelvarStonesInBank = GetBankedCurrencyAmount(CURT_TELVAR_STONES)

	local fillUpNow = (db.fillUpAmount > 0 and currentTelvarStonesInInventory < db.fillUpAmount and currentTelvarStonesInBank > 0) or false

	--Do we have > then 0 Tel'Var stones in our inventory and do we have < then we should fill-up to?
	if currentTelvarStonesInInventory > 0 or fillUpNow then

		--Fill-up the stones, if we are too low in our inventory now?
		if fillUpNow then
			--Get the amount of stones to withdraw from your bank
			local stonesToWithdraw = 0
			--Check if the amount is lower then the stones that should be kept in your inventory
			stonesToWithdraw = db.fillUpAmount - currentTelvarStonesInInventory
			--If we got not enough stones at the bank we use the maximum of the bank
			if stonesToWithdraw > currentTelvarStonesInBank then
				stonesToWithdraw = currentTelvarStonesInBank
			end
			if stonesToWithdraw <= 0 then return false end
			--Withdraw some stones from your bank to your inventory now
			WithdrawTelvarStonesFromBank(stonesToWithdraw)

		--Deposit some stones
		else
			--Get the amount of stones to deposit to your bank
			local stonesToDeposit = 0
			--Keep some stones left in the inventory to get a multiplier?
			if db.keepAmount > 0 and currentTelvarStonesInInventory >= db.keepAmount then
				stonesToDeposit = currentTelvarStonesInInventory - db.keepAmount
			elseif db.keepAmount > 0 and currentTelvarStonesInInventory < db.keepAmount then
				stonesToDeposit = 0
			else
				--Deposit all stones in your inventory to your bank
				stonesToDeposit = currentTelvarStonesInInventory
			end
			--Any stones to deposit?
			if stonesToDeposit == 0 then return false end
			--Check if the amount of stones left in your inventory would be less, then the "fill-up" from the bank
			--and recalculate then to the "fill-up" value
			local stonesLeftInInventoryAfter = currentTelvarStonesInInventory - stonesToDeposit
			if db.fillUpAmount > 0 and db.fillUpAmount > stonesLeftInInventoryAfter then
				local stonesDiff = db.fillUpAmount - stonesLeftInInventoryAfter
				if stonesDiff ~= 0 then
					stonesToDeposit = stonesToDeposit - stonesDiff
				end
			end
			--If we got not enough stones at the inventory we use the maximum of the inventory
			if stonesToDeposit > currentTelvarStonesInInventory then
				stonesToDeposit = currentTelvarStonesInInventory
			end
			--Deposit some stones?
			if stonesToDeposit <= 0 then return false end
			--Deposit the Tel'Var stones to your bank now
			DepositTelvarStonesIntoBank(stonesToDeposit)
		end

	end

end

local function RefreshTelVarHUD()
	
	if GetCurrentMapIndex() == 26 then -- Sewers returns nil
	
		local baseMultiplier = GetTelvarStoneMultiplier(GetTelvarStoneMultiplierThresholdIndex())
		local realMultiplier = baseMultiplier + (actualDistrictBonus / 100) * baseMultiplier
		
		local wholePart = zo_floor(realMultiplier)
		local fractionalPart = zo_round(zo_mod(realMultiplier, 1.0)*100)
		HUD_TELVAR_METER.multiplierWholePart:SetText(zo_strformat(SI_TELVAR_HUD_MULTIPLIER_VALUE_WHOLE, wholePart))
		HUD_TELVAR_METER.multiplierFractionalPart:SetText(zo_strformat(SI_TELVAR_HUD_MULTIPLIER_VALUE_FRACTION, string.format("%02d", fractionalPart)))
		
		return true
		
	end
	
end

local function OnPlayerActivated()

	if IsInImperialCity() then
		
		actualDistrictBonus = 0
		for keepId in pairs(ICKeeps) do
			local owningAlliance = GetKeepAlliance(keepId, BGQUERY_LOCAL)
			if owningAlliance == playerAlliance then
				actualDistrictBonus = actualDistrictBonus + GetDistrictOwnershipTelVarBonusPercent(keepId, BGQUERY_LOCAL)
			end
			ICKeeps[keepId] = owningAlliance
		end
		
		RefreshTelVarHUD()
	
	end

end

-- Called by EVENT_OBJECTIVE_CONTROL_STATE
local function OnObjectiveControlState(_, objectiveKeepId, _, _, _, _, objectiveControlEvent, _, objectiveParam1)
	
	if IsInImperialCity() and ICKeeps[objectiveKeepId] then
		if objectiveControlEvent == OBJECTIVE_CONTROL_EVENT_CAPTURED or objectiveControlEvent == OBJECTIVE_CONTROL_EVENT_RECAPTURED then
			if objectiveParam1 == playerAlliance then
				actualDistrictBonus = actualDistrictBonus + GetDistrictOwnershipTelVarBonusPercent(objectiveKeepId, BGQUERY_LOCAL)
			elseif ICKeeps[objectiveKeepId] == playerAlliance then
				actualDistrictBonus = actualDistrictBonus - GetDistrictOwnershipTelVarBonusPercent(objectiveKeepId, BGQUERY_LOCAL)
			end
			
			ICKeeps[objectiveKeepId] = objectiveParam1
			RefreshTelVarHUD()
			
		end
	end
	
end

-- Initialises the settings and menu
local function OnAddonLoaded(_, addonName)

	--Protect
	if addonName == myAddon.name then
	
		local LAM = LibStub('LibAddonMenu-2.0')
		
		--Default values for the SavedVariables
		local defaults = {
			keepAmount		= 0,
			fillUpAmount	= 0,
		}

		-- Fetch the saved variables
		db = ZO_SavedVars:NewAccountWide(myAddon.savedVarsName, myAddon.savedVarsVersion, nil, defaults)

		-- Create control panel
		local panelData = {
			type = "panel",
			name = GetString(TRTS_ADDON_NAME),
			displayName = ZO_HIGHLIGHT_TEXT:Colorize(GetString(TRTS_ADDON_NAME)),
			author = myAddon.author,
			version = myAddon.optionsVersion,
			registerForRefresh = true,
			registerForDefaults = true,
			website = myAddon.website,
		}

		LAM:RegisterAddonPanel("RollingStonesOptions", panelData)

		--The options in the LAM 2.0 menu
		local optionsTable = {
			{
				type = 'description',
				text = GetString(TRTS_ADDON_DESCRIPTION),
			},

			--==============================================================================
			--BANK OPTIONS
			{
				type = 'header',
				name = GetString(TRTS_OPTIONS_HEADER_BANK_DEPOSIT),
			},
			{
				type = "slider",
				name = GetString(TRTS_OPTIONS_KEEP_STONES),
				tooltip = GetString(TRTS_OPTIONS_KEEP_STONES_TT),
				min = 0,
				max = 10000,
				step = 100,
				getFunc = function() return db.keepAmount end,
				setFunc = function(newValue) db.keepAmount = newValue end,
				default = db.keepAmount,
			},
			--INVENTORY OPTIONS
			{
				type = 'header',
				name = GetString(TRTS_OPTIONS_HEADER_BANK_WITHDRAW),
			},
			{
				type = "slider",
				name = GetString(TRTS_OPTIONS_FILLUP_STONES),
				tooltip = GetString(TRTS_OPTIONS_FILLUP_STONES_TT),
				min = 0,
				max = 10000,
				step = 100,
				getFunc = function() return db.fillUpAmount end,
				setFunc = function(newValue) db.fillUpAmount = newValue end,
				default = db.fillUpAmount,
			},
		}

		LAM:RegisterOptionControls("RollingStonesOptions", optionsTable)

		-- Set the function to run when bank is opened
		EVENT_MANAGER:RegisterForEvent(myAddon.name, EVENT_OPEN_BANK, RollingStones)
		
		EVENT_MANAGER:RegisterForEvent(myAddon.name, EVENT_PLAYER_ACTIVATED, OnPlayerActivated)
		
		ZO_PreHook(HUD_TELVAR_METER, "UpdateMultiplier", RefreshTelVarHUD)
		
		-- Fire each 0.5s itself
		EVENT_MANAGER:RegisterForEvent(myAddon.name, EVENT_OBJECTIVE_CONTROL_STATE, OnObjectiveControlState)
		
		EVENT_MANAGER:UnregisterForEvent(myAddon.name, EVENT_ADD_ON_LOADED)
	
	end
	
end

-- Register addon
EVENT_MANAGER:RegisterForEvent(myAddon.name, EVENT_ADD_ON_LOADED, OnAddonLoaded)